import os

from flask import Flask
from flask_cors import CORS
from flask_sqlalchemy import SQLAlchemy


from api.settings import get_app_config_engine
from api.util import CustomJSONEncoder

# instantiate the extensions
db = SQLAlchemy()
cors = CORS()


def create_app(script_info=None):

    app = Flask(__name__)
    app.json_encoder = CustomJSONEncoder

    # set config
    config_engine = get_app_config_engine()
    app.config.from_object(config_engine)

    # set up extensions
    db.init_app(app)
    cors.init_app(app)

    #     migrate.init_app(app, db)

    # register blueprints
    from .app import main_blueprint

    app.register_blueprint(main_blueprint)
    #     from project.api.users import users_blueprint
    #     app.register_blueprint(users_blueprint)

    # shell context for flask cli
    app.shell_context_processor(lambda: {"app": app, "db": db})
    return app


# Model import occurs here to allow flask_sqlalchemy to import metadata
import api.models
