import os

# TODO Find an alternative to injection envvars into config.
# TODO convert to user logger instead
if os.getenv("FLASK_ENV") == "development":
    try:
        import dotenv
        # find the file relative to this settings module
        current_dir = os.path.dirname(__file__)
        dotenv.load_dotenv(os.path.join(current_dir, ".env"))
    except Exception as err:
        # TODO add logging here and stop raising error?
        raise err


# TODO Add  postgres params option
def build_postgres_uri(host, user, dbname, passwd=None, port="5432", sslmode=None):
    """Builds a PostgreSQL uri.

    See https://www.postgresql.org/docs/current/libpq-connect.html#LIBPQ-CONNSTRING for additional details.
    """

    if passwd:
        uri = f"postgresql://{user}:{passwd}@{host}:{port}/{dbname}"
    else:
        uri = f"postgresql://{user}@{host}:{port}/{dbname}"
    if sslmode:
        uri = f"{uri}?sslmode={sslmode}"
    return uri


class BaseConfigEngine:
    """Base configuration engine"""
    # Allow atomic construction of database uri
    POSTGRES_HOST = os.getenv("POSTGRES_HOST", default="localhost")
    POSTGRES_USER = os.getenv("POSTGRES_USER", default="postgres")
    POSTGRES_APP_DATABASE = os.getenv("POSTGRES_APP_DATABASE", default="postgres")
    POSTGRES_PASSWORD = os.getenv("POSTGRES_PASSWORD")
    POSTGRES_PORT = os.getenv("POSTGRES_PORT", default="5432")
    POSTGRES_SSL_MODE = os.getenv("POSTGRES_SSL_MODE", default="prefer")
    POSTGRES_PARAMS = os.getenv("POSTGRES_PARAMS")
    SECRET_KEY = os.getenv("SECRET_KEY", "defaultsecretkey")
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    # Allow for direct declaration of database uri
    _SQLALCHEMY_DATABASE_URI = os.getenv("SQLALCHEMY_DATABASE_URI")

    @property
    def SQLALCHEMY_DATABASE_URI(self):  # Note: all caps
        if self._SQLALCHEMY_DATABASE_URI:
            return self._SQLALCHEMY_DATABASE_URI
        else:
            return build_postgres_uri(
                self.POSTGRES_HOST,
                self.POSTGRES_USER,
                self.POSTGRES_APP_DATABASE,
                self.POSTGRES_PASSWORD,
                self.POSTGRES_PORT,
                self.POSTGRES_SSL_MODE,
            )


class DevConfigEngine(BaseConfigEngine):
    DEBUG = True
    SQLALCHEMY_ECHO = True


class TestingConfigEngine(BaseConfigEngine):
    TESTING = True


class ProductionConfigEngine(BaseConfigEngine):
    DEBUG = False


app_settings = {
    "dev": DevConfigEngine,
    "development": DevConfigEngine,
    "test": TestingConfigEngine,
    "testing": TestingConfigEngine,
    "prod": ProductionConfigEngine,
    "production": ProductionConfigEngine,
}


def get_app_config_engine():
    """Returns an instance of the Application Config Engine"""
    config_engine = app_settings[os.getenv("APP_SETTINGS", "dev")]()
    return config_engine
